/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_factory.cpp
 *
 * Implements the cmsg_factory class, which is an Abstract Factory for the cmsg
 * class. Messages can only be created from P4 headers using the
 * cmsg_factory::new_msg() method.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "msg_factory.h"
#include "msg.h"
#include "msg_hello.h"
#include "msg_ping.h"
#include "msg_ping_ack.h"
#include "msg_connect.h"
#include "msg_discover.h"
#include "msg_lka.h"
#include "msg_lka_ack.h"
#include "msg_data.h"
#include "debug.h"

/*****************************************************************************
 * PUBLIC METHODS
 *****************************************************************************/

cmsg *cmsg_factory::new_msg(const p4_header &hdr) const
{
  cdebug::instance() << acquire("msg_factory_debug") << "new_msg" << release();

  switch(hdr.get_type()) {
  case kp4_hello:
    return new cmsg_hello(hdr);
  case kp4_ping:
    return new cmsg_ping(hdr);
  case kp4_ping_ack:
    return new cmsg_ping_ack(hdr);
  case kp4_connect:
    return new cmsg_connect(hdr);
  case kp4_discover:
    return new cmsg_discover(hdr);
  case kp4_lka:
    return new cmsg_lka(hdr);
  case kp4_lka_ack:
    return new cmsg_lka_ack(hdr);
  case kp4_data:
    return new cmsg_data(hdr);
  }

  assert(0);
  return 0;
}


cmsg *cmsg_factory::new_msg(const cmsg &msg) const
{
  cdebug::instance() << acquire("msg_factory_debug")
                     << "new_msg (copy ctor version)"
                     << release();

  switch(msg.hdr.get_type()) {
  case kp4_hello:
    return new cmsg_hello(msg);
  case kp4_ping:
    return new cmsg_ping(msg);
  case kp4_ping_ack:
    return new cmsg_ping_ack(msg);
  case kp4_connect:
    return new cmsg_connect(msg);
  case kp4_discover:
    return new cmsg_discover(msg);
  case kp4_lka:
    return new cmsg_lka(msg);
  case kp4_lka_ack:
    return new cmsg_lka_ack(msg);
  case kp4_data:
    return new cmsg_data(msg);
  }

  assert(0);
  return 0;
}
